<?php
/* **************************************************************************** *

 * Database.php									*
 * @package	Core								*
 * @copyright:	Copyright (c) 2008 Martin Novák. All rights reserved		*
 * @license	GNU/AGPL http://www.gnu.org/licenses/agpl.html			*
 
 * Albireo is free software! You can redistribute it and/or modify it under	*
 * the terms of the GNU Affero General Public License as published		*
 * by the Free Software Foundation, either version 3 of the License, or		*
 * (at your option) any later version.						*

 *	This program is distributed in the hope that it will be useful,		*
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of		*
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the		*
 *	GNU Affero General Public License for more details.			*

 This script description:
 This class was developed for work with MySQL database. Interface works with
 design pattern Singleton. You can access an instance by calling method
 connection.
 This class does not support mysqli interface.
 Example: $number_of_news = Database::connection () -> getResult ( $query );

 Properties:
    none
 	
 Public Methods:
    resource connection ()	// contain an instance of class (Singleton design
    				   pattern)
    resource query ( $query )	// SQL queries for database
    array fetchall_row ( $query )
    				// returns two dimensional array as result of query
    				   (number indexes)
    array fetchall_assoc ( $query )
    				// return two demensional array as result of query
    				   (first dimension contains number index and second
    				   associative names of columns)
    char getResult ( $query )	// returns string as result of query
    array fetch_assoc ( $query )// returns associative array as resutl of query
    boolean is_connected ()	// returns boolean information of connection to
    				   the database
    boolean reconnect ()	// reconnect to the database
    boolean connect_to ()	// connect to the database
    
 * **************************************************************************** */

class Database {

    private $dbh;
    private static $instance = null;

    public static function connection ()
    {
    	if ( self::$instance == null ) {
    	    self::$instance = new Database ();
    	}
    	return self::$instance;
    } // end connection

    private function __construct ()
    {
    	$dbhost = Tools::baseSettings () -> getSetting ( "dbhost" );
    	$user = Tools::baseSettings () -> getSetting ( "user" );
    	$pass = Tools::baseSettings () -> getSetting ( "pass" );
    	$dbname = Tools::baseSettings () -> getSetting ( "dbname" );

        $this -> dbh = @new Mysqli ( $dbhost, $user, $pass, $dbname );
        if ( mysqli_connect_errno () ) {
            $this -> dbh = null;
            Error::ticket () -> notice ( 'cor00010','in '. __FILE__ .' on line '. __LINE__, mysqli_connect_errno () .': '. mysqli_connect_error () );
        } else {
            $this -> dbh -> query ( "SET NAMES 'utf8' COLLATE 'utf8_general_ci';" );
        }
    } // end __construct

    private function __clone ()
    {
    } // end __clone

    public function multi_query ( $query )
    {
        $result = $this -> dbh -> multi_query ( $query );
        if ( $result !== false ) {
            return $result;
        } else {
            return false;
        }
    } // end multi_query

    public function query ( $query )
    {
        if ( $result = $this -> dbh -> query ( $query ) ) {
    	    return $result;
    	} else {
    	    Error::ticket () -> notice ( 'cor00011','in '.__FILE__.' on line '.__LINE__, $query.'<br />MySQL: '. $this -> dbh -> errno .': '.$this -> dbh -> error );
            return false;
    	}
    } // end query

    public function fetchall_row ( $query )
    {
    	if ( !$result = $this -> query ( $query ) ) {
    	    return false;
    	}
    	$retval = array ();
    	while ( $row = $result -> fetch_row () ) {
   	    $retval [] = $row;
  	}
  	return $retval;
    } // end fetchall_row

    public function fetchall_assoc ( $query )
    {
    	if ( !$result = $this -> query ( $query ) ) {
    	    return false;
    	}
    	$retval = array ();
  	while ( $row = $result -> fetch_assoc () ) {
   	    $retval [] = $row;
  	}
  	return $retval;
    } // end fetchall_assoc

    public function getResult ( $query )
    {
        $result = $this -> query ( $query );
        if ( !is_object ( $result ) ) return false;
        $array = $result -> fetch_row ();
        if ( !is_array ( $array ) ) return false;
        return current ( $array );
    } // end getResult

    public function fetch_assoc ( $query )
    {
        $result = $this -> query ( $query );
        if ( !is_object ( $result ) ) return false;
        return $result -> fetch_assoc ();
    } // end fetch_assoc

    public function backup_table ( $table )
    {
    	$rows = "DROP TABLE IF EXISTS `$table`;\n";
    	$create = $this -> fetch_assoc ( "SHOW CREATE TABLE `$table`;" );
    	$rows .= $create [ "Create Table" ] .";\n\n";
        return $rows;
    } // end backup_table

    public function backup_table_contents ( $table ) {
        $new_row = "";
        $rows = "";
    	@set_time_limit ( 0 );
        $contents = $this -> fetchall_row ( "SELECT * FROM `$table`;" );
    	if ( is_array ( $contents ) ) {
    	    foreach ( $contents as $row ) {
    	    	$new_row = "INSERT INTO `$table` VALUES ( ";
    	    	foreach ( $row as $value ) {
    	    	    if ( is_numeric ( $value ) ) {
    	    	    	$new_row .= "$value, ";
    	    	    } elseif ( is_null ( $value ) ) {
                        $new_row .= "NULL, ";
                    } else {
    	    	    	$new_row .= "'". str_replace ( "'", "\'", $value ) ."', ";
    	    	    }
    	    	}
    	    	$rows .= substr ( $new_row, 0, -2 ) ." );\n";
    	    }
    	}
    	return $rows;
    } // end backup_table_contents

    public function backup_trigger ( $trigger )
    {
    	$rows = "DROP TRIGGER IF EXISTS `$trigger`;\n";
        $create = $this -> fetch_assoc ( "SHOW CREATE TRIGGER `$trigger`;" );
    	$rows .= $create [ "SQL Original Statement" ] .";\n";
    	return $rows;
    } // end backup_trigger

    public function backup_database ()
    {
    	$rows = "";
    	@set_time_limit ( 0 );

    	$tables = $this -> fetchall_row ( "SHOW TABLES;" );
        $triggers = $this -> fetchall_row ( "SHOW TRIGGERS;" );

        $rows .= $this -> backup_table ( "users" );
        $rows .= $this -> backup_table ( "usergroups" );
        $rows .= $this -> backup_table ( "directory" );
        $rows .= $this -> backup_table ( "header" );
        $rows .= $this -> backup_table ( "extensions" );

        if ( is_array ( $tables ) ) {
            foreach ( $tables as $table ) {
                if ( $table [ 0 ] != "users" && $table [ 0 ] != "usergroups" &&
                     $table [ 0 ] != "directory" && $table [ 0 ] != "header" && $table [ 0 ] != "extensions" ) {
                    $rows .= $this -> backup_table ( $table [ 0 ] );
                }
            }
        }

        if ( is_array ( $triggers ) ) {
            $rows .= "\n\nDELIMITER //\n\n";
            foreach ( $triggers as $trigger ) {
                $rows .= $this -> backup_trigger ( $trigger [ 0 ] );
                $rows .= "\n//\n\n";
            }
            $rows .= "DELIMITER ;\n\n";
        }

        $rows .= $this -> backup_table_contents ( "users" );
        $rows .= $this -> backup_table_contents ( "usergroups" );
        $rows .= $this -> backup_table_contents ( "directory" );
        $rows .= $this -> backup_table_contents ( "header" );
        $rows .= $this -> backup_table_contents ( "extensions" );

        if ( is_array ( $tables ) ) {
            foreach ( $tables as $table ) {
                if ( $table [ 0 ] != "users" && $table [ 0 ] != "usergroups" &&
                     $table [ 0 ] != "directory" && $table [ 0 ] != "header" && $table [ 0 ] != "extensions" ) {
                    $rows .= $this -> backup_table_contents ( $table [ 0 ] );
                }
            }
        }

        return $rows;
    } // end backup_database

    public function sql_from_file ( $path_to_file, $ignore = true ) // if is ignore true, the function will ignore errors during process
    {
        $handle = @fopen ( $path_to_file, 'r' );
         if ( !$handle ) {
            return false;
        }
        $dbhost = Tools::baseSettings () -> getSetting ( "dbhost" );
    	$user = Tools::baseSettings () -> getSetting ( "user" );
    	$pass = Tools::baseSettings () -> getSetting ( "pass" );
    	$dbname = Tools::baseSettings () -> getSetting ( "dbname" );

        $mysqli = new Mysqli ( $dbhost, $user, $pass, $dbname );
        if ( mysqli_connect_errno () ) {
            return false;
        }
        $mysqli -> query ( "SET NAMES 'utf8' COLLATE 'utf8_general_ci';" );
        $result = array ( "errors" => 0, "ok" => 0 );
        $delimiter = ";";
        $delimiter_length = 1;
        $sql = "";
        @set_time_limit ( 0 );
        while ( !feof ( $handle ) ) {
            $line = fgets ( $handle );
            if ( substr ( ltrim ( $line ), 0, 2 ) !== '--' ) {
                $sql .= $line;
                if ( strtolower ( substr ( ltrim ( $line ), 0, 9 ) ) !== 'delimiter' ) {
                    if ( substr ( rtrim ( $line ), -$delimiter_length ) === $delimiter ) {
                        if ( $delimiter != ";" ) $sql = substr ( rtrim ( $sql ), 0, -$delimiter_length );
                        if ( $mysqli -> multi_query ( $sql ) ) {
                            do {
                                $mysqli -> use_result ();
                                if ( $mysqli -> errno == 0 ) {
                                    $result [ "ok" ] = $result [ "ok" ] + 1;
                                } else {
                                    $result [ "errors" ] = $result [ "errors" ] + 1;
                                }
                            } while ( $mysqli -> next_result () );
                        } elseif ( $ignore == false ) {
                            return false;
                        } else {
                            $result [ "errors" ] = $result [ "errors" ] + 1;
                        }
                        $sql = "";
                    }
                } else {
                    $delimiter = substr ( trim ( $line ), 10 );
                    $delimiter_length = strlen ( $delimiter );
                    $sql = "";
                }
            }
        }

        fclose ( $handle );
    	return $result;
    } // end sql_from_file

    public function is_connected ()
    {
    	return is_object ( $this -> dbh );
    } // end is_connected

    public function reconnect ()
    {
    	if ( self::$instance = new Database () ) {
    	    return true;
    	} else {
    	    return false;
    	}
    } // end reconnect

    public function connect_to ( $dbhost, $user, $pass, $dbname )
    {
        $this -> dbh = new Mysqli ( $dbhost, $user, $pass, $dbname );
        if ( mysqli_connect_errno () ) {
            return false;
        } else {
            $this -> dbh -> query ( "SET NAMES 'utf8' COLLATE 'utf8_general_ci';" );
            return true;
        }
    } // end connect_to

}
?>